#!/usr/bin/python3

from configparser import ConfigParser
import datetime
import errno
import getopt
import json
import logging
import math
import os
import pwd
import sqlite3
import sys
import threading
from threading import current_thread
import time

#config file full path
config_path = '/usr/local/safesquid/api/config.ini';

# set config initialization
config = ConfigParser()

# parse existing file
config.read(config_path)
# read values from a section
log_path = config.get('log_info', 'log_path')
log_file = config.get('log_info', 'log_file')
report_dir = config.get('report_path', 'report_dir')
db_resource = config.get('database', 'resource_db')
user = config.get('permission', 'user')
group = config.get('permission', 'group')

# logs to be written to file
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

# create a file handler
log_file_path = log_path + log_file
handler = logging.FileHandler(log_file_path)
handler.setLevel(logging.INFO)


# create a logging format
formatter = logging.Formatter('%(asctime)s - %(name)s - [ %(funcName)s():%(lineno)3s ] - %(levelname)s - %(message)s')
handler.setFormatter(formatter)

# add the handlers to the logger
logger.addHandler(handler)

#global variables to be used
from_date = ''
to_date = ''
total_days = ''
pattern = '%d.%m.%Y %H:%M:%S'

def epoch_time(date_time=None):
    if date_time is None:
        date_time = time.strftime("%d.%m.%Y %H:%M:%S")

    pattern = '%d.%m.%Y %H:%M:%S'
    epoch = int(time.mktime(time.strptime(date_time, pattern)))
    return epoch

def getdays(fromdata, todate):
    date_format = "%d.%m.%Y"
    a = datetime.datetime.strptime(fromdata, date_format)
    b = datetime.datetime.strptime(todate, date_format)

    delta = b - a
    return delta.days

def reverse_lookup(array_name,value):
    try:
        return list(array_name.keys())[list(array_name.values()).index(value)]
    except ValueError:
        return None

#fetch command line argument so that report can be generated.
def arguments():
    global from_date, to_date, total_days
    def usage():
        print ('Usage : ./app_handler.py -t <day|hrs|mon>                               -- Generate for particular type')
        print ('        ./app_handler.py -d <date>                                      -- Generate for particular date')
        print ('        ./app_handler.py -s <start date> -e <end date>                  -- Generate between time range')
        print ('Date formatte will be in dd.mm.yyyy Eg : 03.08.1990')
        print ('If you are using time range. Try Generate reports in packs of 7 days ')

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hs:e:t:d:")
    except getopt.GetoptError as err:
        print (str(err))
        usage()
        sys.exit(2)

    if (len(opts) == 0):
        logger.error('No arg passed')
        usage()
        sys.exit(2)
    else:
        logger.info('total arg passed: %s', len(opts))

        for opt, arg in opts:
            if opt == '-h':
                usage()
                sys.exit(2)
            elif opt in ("-s"):
                from_date = time.strftime(arg + " 00:00:00")
                logger.info('value passed for start date is %s', from_date)
            elif opt in ("-e"):
                to_date = time.strftime(arg + " 00:00:00")
                logger.info('value passed for end date is %s', to_date)
            elif opt in ("-t"):
                from_date = time.strftime(arg + " 00:00:00")
                to_date = time.strftime(arg + " 23:59:59")
            elif opt in ("-d"):
                from_date = time.strftime(arg + " 00:00:00")
                to_date = time.strftime(arg + " 23:59:59")

    if not from_date:
        lastHourDateTime = datetime.datetime.now() - datetime.timedelta(hours=1)
        from_date = lastHourDateTime.strftime("%d.%m.%Y %H:00:00")
        del lastHourDateTime

    if not to_date:
        to_date = time.strftime("%d.%m.%Y %H:00:00")

    logger.info('value passed for start date id %s and end date is %s', from_date, to_date)

    fd_ = from_date.split(' ')
    td_ = to_date.split(' ')

    total_days = getdays(fd_[0], td_[0])

    if total_days < 0:
        logger.error('end date defination is wrong')
        print ('Incorrect end date . Start date [' + from_date + '] is greater than end date [' + to_date + ']')
        sys.exit(2)
    else:
        total_days = total_days  + 1

    logger.info('Report to be generated for %s days', total_days)

    del fd_
    del td_


def convert_size(size_bytes):
    if not size_bytes:
        return '0B'

    if (size_bytes == 0):
        return '0B'

    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return '%s %s' % (s, size_name[i])

def getSize(filename):
    try:
        st = os.stat(filename)
        return convert_size(st.st_size)
    except (OSError, ValueError) as e:
        logger.error(str(e))
        print (filename + ' filename is not available')
        sys.exit(2)

def write_file(file_name, data):
    f = open(file_name, 'w')
    f.write(data)
    f.close()
    
def delete_file(path):
    try:
        if os.path.isfile(path):
            os.remove(path) 
        elif os.path.isdir(path):
            shutil.rmtree(path) 
    except OSError as e: 
        print ("Error: %s - %s." % (e.filename, e.strerror))

def check_directory(path):
    if not os.path.exists(path):
        try:
            uid, gid = pwd.getpwnam(user).pw_uid, pwd.getpwnam(group).pw_uid
            logger.info('creating directory %s with [%s:%s]', path, user, group)
            os.makedirs(path)
            os.chown(path, uid, gid)
        except OSError as exception:
            if exception.errno != errno.EEXIST:
                logger.error('creating directory %s failed', path)
                raise

class GetQueryThread(threading.Thread):
    def __init__(self, parent_thd, path, content):
        self.url = content[0]
        self.tag = content[1]
        self.parent_thd = parent_thd
        self.path = path
        threading.Thread.__init__(self)

    def run(self):
        start_s = time.time()
        thread_id = current_thread().name

        conn = sqlite3.connect(db_resource)

        try:
            conn.row_factory = sqlite3.Row
            curs = conn.cursor()
            logger.info('[%s:%s] Quering for [%s] : %s', self.parent_thd, thread_id, self.tag, self.url)
            curs.execute(self.url)
            recs = curs.fetchall()
        except sqlite3.OperationalError as er:
            logger.error('[%s:%s] error: %s', self.parent_thd, thread_id, er)
        except sqlite3.Error as er:
            logger.error('[%s:%s] error: %s', self.parent_thd, thread_id, er.message)

        conn.close()

        try:
            rows = [dict(rec) for rec in recs]
            logger.info('[%s:%s] Completed query [%s] processing in : %s', self.parent_thd, thread_id, self.tag, (time.time()-start_s))
            write_file(self.path + "/." + self.tag, json.dumps(rows))
            
        except UnboundLocalError as er:            
            logger.error('[%s:%s] error: %s', self.parent_thd, thread_id, er)



def generate_report(thread_id, from_, to_, path_):

    f_time = str(epoch_time(from_))
    t_time = str(epoch_time(to_))

    total_1 = "select count(DISTINCT L) as bu,count(D) as rq,sum(G) as tb from ( select L,D,G from main order by D ) where D between " + f_time + " and " + t_time + " ;"
    total_bl = "select count(D) as bls from (select D from main where F=403 ) where D between " + f_time + " and " + t_time + " ;"
    total_br = "select count(D) as brs from (select D from main where F NOT IN(503,500,407,403) ) where D between " + f_time + " and " + t_time + " ;"
    u_list = "select DISTINCT users_master.A as users from (select L from main where D between " + f_time + " and " + t_time + "),users_master where L=users_master.B order by users;"
    d_list = "select req_subdomains_master.A as urls from (select X from main where D between " + f_time + " and " + t_time + "),req_subdomains_master where X=req_subdomains_master.B group by X order by urls;"
    c_list = "select DISTINCT c_categories_master.A as cat from (select AF from main where D between " + f_time + " and " + t_time + " ),c_categories_master where AF=c_categories_master.B order by cat;"
    u_summary = "select users_master.A as users, rq, bt,ul,dw, bw from (select L as user, count(D) as rq, sum(E) as bt, sum(H) as ul, sum(I) as dw, sum(G) as bw from (select D,L,G,E,H,I from main order by D) where D between " + f_time + " and " + t_time + " group by L order by bw DESC),users_master where user=users_master.B;"
    u_dn_num = "select users_master.A as users, dn from(select count(DISTINCT N) as dn, L from main where F=403 and D between " + f_time + " and " + t_time + " group by L),users_master where L=users_master.B;"
    u_al_num = "select users_master.A as users, br from(select count(DISTINCT X) as br,L from main where F NOT IN (503,500,407,403) and D between " + f_time + " and " + t_time + " group by L),users_master where L=users_master.B;"
    c_summary = "select c_categories_master.A as cat, rq, bt,ul,dw, bw from (select AF as cats, count(D) as rq, sum(E) as bt, sum(H) as ul, sum(I) as dw, sum(G) as bw from (select D,AF,G,E,H,I from main order by D) where D between " + f_time + " and " + t_time + " group by AF order by bw DESC),c_categories_master where cats=c_categories_master.B order by cat;"
    c_dn_num = "select c_categories_master.A as cat, dn from(select count(DISTINCT N) as dn, AF from main where F=403 and D between " + f_time + " and " + t_time + " group by AF),c_categories_master where AF=c_categories_master.B order by dn DESC;"
    c_al_num = "select c_categories_master.A as cat, br from(select count(DISTINCT X) as br,AF from main where F NOT IN (503,500,407,403) and D between " + f_time + " and " + t_time + " group by AF),c_categories_master where AF=c_categories_master.B order by br DESC;"
    u_dm_each = "select req_subdomains_master.A as urls, users_master.A as users, count(X) as rq, sum(G) as bw from (select X,L,G from main where F NOT IN (503,500,407,403) and  D between " + f_time + " and " + t_time + "),users_master,req_subdomains_master where L=users_master.B and X=req_subdomains_master.B group by X,L order by bw DESC;"
    u_c_each = "select c_categories_master.A as cat, users_master.A as users, count(AF) as rq, sum(G) as bw from (select AF,L,G from main where F NOT IN (503,500,407,403) and  D between " + f_time + " and " + t_time + "),users_master,c_categories_master where L=users_master.B and AF=c_categories_master.B group by AF,L order by bw DESC;"
    c_dm_each = "select req_subdomains_master.A as urls, count(X) as rq, sum(G) as bw,  c_categories_master.A as cat from (select X,AF,G from main where F NOT IN (503,500,407,403) and  D between " + f_time + " and " + t_time + "),c_categories_master,req_subdomains_master where AF=c_categories_master.B and X=req_subdomains_master.B group by X,AF order by bw DESC;"
    u_dn_fil = "select url_master.A as urls, users_master.A as users, count(N) as rq, sum(G) as bw, filter_name_master.A as fil from (select N,L,G,R from main where F=403 and D between " + f_time + " and " + t_time + " ),users_master,url_master,filter_name_master where L=users_master.B and N=url_master.B and R=filter_name_master.B group by N,L order by bw DESC;"
    total_day_bw = "select strftime('%H',datetime(D, 'unixepoch', 'localtime')) as interval, count(D) as rq, sum(G) as bw from (select D,G from main) where D between " + f_time + " and " + t_time + " group by interval order by bw DESC;"    
    hour_dm = "select interval, req_subdomains_master.A as urls, rq, bw from (select strftime('%H',datetime(D,'unixepoch','localtime')) as interval, X as domain, count(X) as rq,sum(G) as bw from (select D,X,G from main order by D) where D between " + f_time + " and " + t_time + " group by interval,X order by interval),req_subdomains_master where domain = req_subdomains_master.B;"
    hour_u = "select interval, users_master.A as users, rq, bw from (select strftime('%H',datetime(D, 'unixepoch', 'localtime')) as interval, L as user, count(L) as rq,sum(G) as bw from (select D,L,G from main order by D) where D between " + f_time + " and " + t_time + " group by interval, L order by interval),users_master where user = users_master.B;"
    hour_c = "select interval, c_categories_master.A as cat, rq, bw from (select strftime('%H',datetime(D, 'unixepoch', 'localtime')) as interval, AF as cats, count(AF) as rq,sum(G) as bw from (select D,AF,G from main order by D) where D between " + f_time + " and " + t_time + " group by interval, AF order by interval),c_categories_master where cats = c_categories_master.B;"
    total_dn_fil = "select url_master.A as urls, rq, bw,filter_name_master.A as fil,c_profiles_master.A as pro from(select N, count(N) as rq, sum(G) as bw,R,AJ from main where F=403 and D between " + f_time + " and " + t_time + " group by N,AJ,R),url_master,filter_name_master,c_profiles_master where N=url_master.B and R=filter_name_master.B and AJ=c_profiles_master.B order by bw DESC;"
    total_al_dm = "select req_subdomains_master.A as urls, rq, bw, c_profiles_master.A as pro from(select X, count(X) as rq, sum(G) as bw,AJ from main where F NOT IN (503,500,407,403) and D between " + f_time + " and " + t_time + " group by X,AJ),req_subdomains_master,c_profiles_master where X=req_subdomains_master.B and AJ=c_profiles_master.B order by bw DESC;"
    c_dn = "select url_master.A as urls,c_categories_master.A as cat from(select N, AF from main where F=403 and D between " + f_time + " and " + t_time + " group by N,AF ),url_master, c_categories_master where N =url_master.B and AF =c_categories_master.B;"
    dm_dc = "select req_subdomains_master.A as urls, c_dc_types_master.A as dc, count(X) as rq, sum(G) as bw from (select X,AI,G from main where D between " + f_time + " and " + t_time + "),c_dc_types_master,req_subdomains_master where AI=c_dc_types_master.B and X=req_subdomains_master.B group by X,AI order by urls;"
    dc_summary = "select c_dc_types_master.A as dc, rq,bw from (select AI as dct, count(D) as rq,sum(G) as bw from (select D,G,AI from main order by D) where D between " + f_time + " and " + t_time + " group by AI order by bw DESC),c_dc_types_master where dct=c_dc_types_master.B;"
    uc_summary = "select c_uc_types_master.A as uc,rq,bw from (select AH as uct,count(D) as rq,sum(G) as bw from (select D,G,AH from main order by D) where D between " + f_time + " and " + t_time + " group by AH order by bw DESC),c_uc_types_master where uct=c_uc_types_master.B;"
    uc_dm = "select req_subdomains_master.A as urls, c_uc_types_master.A as uc, count(X) as rq, sum(G) as bw from (select X,AH,G from main where D between " + f_time + " and " + t_time + "),c_uc_types_master,req_subdomains_master where AH=c_uc_types_master.B and X=req_subdomains_master.B group by X,AH order by urls;"

    queries = [[total_1, "total_1"], [total_bl, "total_bl"], [total_br, "total_br"], [u_list, "u_list"], [d_list, "d_list"], [c_list, "c_list"], [u_summary, "u_summary"], [u_dn_num, "u_dn_num"], [u_al_num, "u_al_num"], [c_summary, "c_summary"], [c_dn_num, "c_dn_num"], [c_al_num, "c_al_num"], [u_dm_each, "u_dm_each"], [u_c_each, "u_c_each"], [c_dm_each, "c_dm_each"], [u_dn_fil, "u_dn_fil"], [total_day_bw, "total_day_bw"], [hour_dm, "hour_dm"], [hour_u, "hour_u"], [hour_c, "hour_c"], [total_dn_fil, "total_dn_fil"], [total_al_dm, "total_al_dm"], [c_dn, "c_dn"], [dm_dc, "dm_dc"], [dc_summary, "dc_summary"], [uc_summary, "uc_summary"], [uc_dm, "uc_dm"]]

    start2 = time.time()
    threadsQ = []

    for url in queries:
        t2 = GetQueryThread(thread_id, path_, url)
        threadsQ.append(t2)
        t2.start()
    for t2 in threadsQ:
        t2.join()

    logger.info('Completed query process in : %s', (time.time()-start2))
    return

class GetDayThread(threading.Thread):
    def __init__(self, content):
        self.from_ = content[0]
        self.to_ = content[1]
        self.path_ = content[2]
        threading.Thread.__init__(self)

    def run(self):
        start_s = time.time()
        thread_id = current_thread().name
        logger.info('[%s] check for %s', thread_id, self.from_)
        generate_report(thread_id, self.from_, self.to_, self.path_)
        logger.info('[%s] Completed [%s]:[%s] days processing in : %s', thread_id, self.from_, self.to_, (time.time()-start_s))

if __name__ == "__main__":
    days_path = []
    logger.info('Starting report generations')
    arguments()
    logger.info('Generating report from %s [%s]', db_resource, getSize(db_resource))

    t_ = to_date

    start = time.time()
    threads = []
    for x in range(total_days):
        lastHourDateTime = datetime.datetime.strptime(t_, pattern) - datetime.timedelta(days=x)
        from_date = lastHourDateTime.strftime("%d.%m.%Y 00:00:00")
        to_date = lastHourDateTime.strftime("%d.%m.%Y 23:59:59")

        report_str_ = to_date.split(' ')
        report_str = report_str_[0].split('.')

        report_path = report_dir + '/' + report_str[2] + '/' + report_str[1] + '/' + report_str[0]
        check_directory(report_path)
        days_path.append(report_path)

        t = GetDayThread([from_date, to_date, report_path])
        threads.append(t)
        t.start()

    for t in threads:
        t.join()

    logger.info('Completed process in : %s', (time.time()-start))

    logger.info('starting computing');

    total_queries_ = config.get('query_match', 'queries')
    total_queries = total_queries_.split(',')
    
    del total_queries_
    
    for url_path in days_path:
        data_container = {}
        report_container = {}
        report_path = url_path
        
        for file_name in total_queries:
            fetch_file = report_path + "/." + str(file_name)
            
            try:
                with open(fetch_file, 'r') as myfile:
                    data = myfile.read()
                    data_container[file_name] = data            
                    delete_file(fetch_file)
            except FileNotFoundError as er:        
                logger.error('%s', er)
        
        dashboard_ = config.get('query_match', 'dashboard')
        dashboard = dashboard_.split(',')
        
        del dashboard_
        
        ''' dashboard  '''
        total_1 = json.loads(data_container['total_1'])
        total_bl = json.loads(data_container['total_bl'])
        total_br = json.loads(data_container['total_br'])
        total_day_bw = json.loads(data_container['total_day_bw'])
        
        report_container['bw'] = total_1[0]['tb']
        report_container['rq'] = total_1[0]['rq']
        report_container['us'] = total_1[0]['bu']
        report_container['bl'] = total_bl[0]['bls']
        report_container['al'] = total_br[0]['brs']
        report_container['ha'] = total_day_bw
        
        del total_1
        del total_bl
        del total_br
        del total_day_bw
        
        write_file(report_path + "/summary.json", json.dumps(report_container)) 
        
        report_container = {}
        
        ''' userlist database '''
        
        fetch_user = report_dir + "/user.json" 
        try:
            with open(fetch_user, 'r') as myfile:
                user_list = json.loads(myfile.read())                                
            cnt = len(user_list) + 1   
        except FileNotFoundError:        
            user_list = {}
            cnt = 1
            
        userlist = json.loads(data_container['u_list'])
        
        for s in userlist:
            current_user = s['users']        
            user_identity = "u" + str(cnt)
            userlist_len = len(user_list)
            
            if current_user == '-' :
                current_user = "unknown"
            
            if userlist_len != 0:        
                rl = reverse_lookup(user_list,current_user)            
            else:
                rl = None
                
            if rl is  None:                    
                user_list[user_identity] = current_user            
                cnt = cnt + 1
            
        report_container = user_list        
        write_file(fetch_user, json.dumps(report_container))
    
        del fetch_user
        
        report_container = {}
        
        ''' url database '''
        
        fetch_user = report_dir + "/sites.json" 
        try:
            with open(fetch_user, 'r') as myfile:
                site_list = json.loads(myfile.read())                                
            cnt = len(site_list) + 1   
        except FileNotFoundError:        
            site_list = {}            
            cnt = 1
        
        new_site_list = []
        domainlist = json.loads(data_container['d_list'])
        urllist = json.loads(data_container['total_dn_fil'])
        
        for s in domainlist:
            current_site = s['urls']        
            site_identity = "s" + str(cnt)
            sitelist_len = len(site_list)
            
            if current_site == '-' :
                current_site = "unknown"
            
            if sitelist_len != 0:        
                rl = reverse_lookup(site_list,current_site)            
            else:
                rl = None
                
            if rl is  None:                    
                site_list[site_identity] = current_site 
                new_site_list.append(site_identity)
                cnt = cnt + 1
            
        for s in urllist:
            current_site = s['urls']        
            site_identity = "s" + str(cnt)
            sitelist_len = len(site_list)
            
            if current_site == '-' :
                current_site = "unknown"
            
            if sitelist_len != 0:        
                rl = reverse_lookup(site_list,current_site)            
            else:
                rl = None
                
            if rl is  None:                    
                site_list[site_identity] = current_site            
                new_site_list.append(site_identity)
                cnt = cnt + 1
            
        report_container = site_list        
        write_file(fetch_user, json.dumps(report_container))
        write_file(report_path + "/newsites.json", json.dumps(new_site_list)) 
    
        report_container = {}
        container_json = []
        
        ''' allowed source '''
        alloweddomain = json.loads(data_container['total_al_dm'])
        
        for s in alloweddomain:
            current_site = s['urls']                    
            site_id = reverse_lookup(site_list,current_site)            
            container_json.append({'url':site_id, 'bw': s["bw"], 'rq': s["rq"], 'pro': s["pro"]})
            
        write_file(report_path + "/allowed.json", json.dumps(container_json))
        
        container_json = []
        
        del alloweddomain
        
        ''' Denied source '''
        denieddomain = json.loads(data_container['total_dn_fil'])
        
        for s in denieddomain:
            current_site = s['urls']                    
            site_id = reverse_lookup(site_list,current_site)               
            container_json.append({'url':site_id, 'bw': s["bw"], 'rq': s["rq"],'fn': s["fil"], 'pro': s["pro"]})
            
        write_file(report_path + "/denied.json", json.dumps(container_json))
        
        del denieddomain
        container_json = []
        
        ''' User report details '''
        user_summary = json.loads(data_container['u_summary'])
        
        for s in user_summary:
            current_user = s['users']  
            
            if current_user == '-' :
                current_user = "unknown"
            
            user_id = reverse_lookup(user_list,current_user)
         
            try:
                report_container[user_id]
            except KeyError:
                report_container[user_id] = {}
        
            report_container[user_id]['bw'] = s['bw']
            report_container[user_id]['tm'] = s['bt']
            report_container[user_id]['rq'] = s['rq']
            report_container[user_id]['dw'] = s['dw']
            report_container[user_id]['ul'] = s['ul']
            report_container[user_id]["hour"] = []
            report_container[user_id]["allowed"] = []
            report_container[user_id]["blocked"] = []

        user_denied = json.loads(data_container['u_dn_num']) 
        
        for s in user_denied:
            current_user = s['users'] 
            if current_user == '-' :
                current_user = "unknown"
            user_id = reverse_lookup(user_list,current_user)
            
            report_container[user_id]['bl'] = s['dn']
            
        user_allowed = json.loads(data_container['u_al_num'])     
            
        for s in user_allowed:
            current_user = s['users']  
            if current_user == '-' :
                current_user = "unknown"
            user_id = reverse_lookup(user_list,current_user)
            
            report_container[user_id]['al'] = s['br']
        
        user_hour = json.loads(data_container['hour_u'])     
            
        for s in user_hour:
            current_user = s['users']  
            if current_user == '-' :
                current_user = "unknown"
                
            user_id = reverse_lookup(user_list,current_user)
            
            report_container[user_id]['hour'].append({'int':s["interval"], 'bw': s["bw"], 'rq': s["rq"]})
        
        del user_hour
        
        user_denied_sites = json.loads(data_container['u_dn_fil'])
        
        for s in user_denied_sites:
            current_user = s['users']  
            if current_user == '-' :
                current_user = "unknown"
                
            user_id = reverse_lookup(user_list,current_user)    
            
            current_url = s['urls']  
            if current_url == '-' :
                current_url = "unknown"
                
            url_id = reverse_lookup(site_list,current_url)            
            report_container[user_id]['blocked'].append({'bl':url_id, 'bw': s["bw"], 'fn': s["fil"], 'rq': s["rq"]})
        
        del user_denied_sites
        
        user_allowed_sites = json.loads(data_container['u_dm_each'])
        
        for s in user_allowed_sites:
            current_user = s['users']  
            if current_user == '-' :
                current_user = "unknown"
                
            user_id = reverse_lookup(user_list,current_user)    
            
            current_url = s['urls']  
            if current_url == '-' :
                current_url = "unknown"
                
            url_id = reverse_lookup(site_list,current_url)            
            report_container[user_id]['allowed'].append({'al':url_id, 'bw': s["bw"], 'rq': s["rq"]})
        
        del user_allowed_sites
        
        write_file(report_path + "/report.json", json.dumps(report_container))
        
    logger.info('ends computing')
    logger.info('Completed report generations')
